/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ITelemetryLogger } from "@fluidframework/common-definitions";
import { IFluidHandle } from "@fluidframework/core-interfaces";
import { IChannelAttributes, IFluidDataStoreRuntime, IChannelStorageService, IChannelServices } from "@fluidframework/datastore-definitions";
import { ISequencedDocumentMessage } from "@fluidframework/protocol-definitions";
import { IGarbageCollectionData, ISummaryTreeWithStats } from "@fluidframework/runtime-definitions";
import { EventEmitterWithErrorHandling } from "@fluidframework/telemetry-utils";
import { IFluidSerializer } from "./serializer";
import { SummarySerializer } from "./summarySerializer";
import { ISharedObject, ISharedObjectEvents } from "./types";
/**
 *  Base class from which all shared objects derive
 */
export declare abstract class SharedObjectCore<TEvent extends ISharedObjectEvents = ISharedObjectEvents> extends EventEmitterWithErrorHandling<TEvent> implements ISharedObject<TEvent> {
    id: string;
    protected runtime: IFluidDataStoreRuntime;
    readonly attributes: IChannelAttributes;
    /**
     * @param obj - The thing to check if it is a SharedObject
     * @returns Returns true if the thing is a SharedObject
     */
    static is(obj: any): obj is SharedObjectCore;
    get ISharedObject(): this;
    get IChannel(): this;
    get IFluidLoadable(): this;
    /**
     * The handle referring to this SharedObject
     */
    readonly handle: IFluidHandle;
    /**
     * Telemetry logger for the shared object
     */
    protected readonly logger: ITelemetryLogger;
    /**
     * Connection state
     */
    private _connected;
    /**
     * Services used by the shared object
     */
    private services;
    /**
     * True if the dds is bound to its parent.
     */
    private _isBoundToContext;
    /**
     * Tracks error that closed this object.
     */
    private closeError?;
    /**
     * Gets the connection state
     * @returns The state of the connection
     */
    get connected(): boolean;
    /**
     * @param id - The id of the shared object
     * @param runtime - The IFluidDataStoreRuntime which contains the shared object
     * @param attributes - Attributes of the shared object
     */
    constructor(id: string, runtime: IFluidDataStoreRuntime, attributes: IChannelAttributes);
    /**
     * Marks this objects as closed. Any attempt to change it (local changes or processing remote ops)
     * would result in same error thrown. If called multiple times, only first error is remembered.
     * @param error - error object that is thrown whenever an attempt is made to modify this object
     */
    private closeWithError;
    /**
     * Verifies that this object is not closed via closeWithError(). If it is, throws an error used to close it.
     */
    private verifyNotClosed;
    /**
     * Event listener handler helper that can be used to react to exceptions thrown from event listeners
     * It wraps error with DataProcessingError, closes this object and throws resulting error.
     * See closeWithError() for more details
     * Ideally such situation never happens, as consumers of DDS should never throw exceptions
     * in event listeners (i.e. catch any of the issues and make determination on how to handle it).
     * When such exceptions propagate through, most likely data model is no longer consistent, i.e.
     * DDS state does not match what user sees. Because of it DDS moves to "corrupted state" and does not
     * allow processing of ops or local changes, which very quickly results in container closure.
     */
    private eventListenerErrorHandler;
    private attachListeners;
    /**
     * A shared object, after construction, can either be loaded in the case that it is already part of
     * a shared document. Or later attached if it is being newly added.
     * @param services - Services used by the shared object
     */
    load(services: IChannelServices): Promise<void>;
    /**
     * Initializes the object as a local, non-shared object. This object can become shared after
     * it is attached to the document.
     */
    initializeLocal(): void;
    /**
     * {@inheritDoc (ISharedObject:interface).bindToContext}
     */
    bindToContext(): void;
    /**
     * {@inheritDoc (ISharedObject:interface).connect}
     */
    connect(services: IChannelServices): void;
    /**
     * {@inheritDoc (ISharedObject:interface).isAttached}
     */
    isAttached(): boolean;
    /**
     * {@inheritDoc (ISharedObject:interface).getAttachSummary}
     */
    abstract getAttachSummary(fullTree?: boolean, trackState?: boolean): ISummaryTreeWithStats;
    /**
     * {@inheritDoc (ISharedObject:interface).summarize}
     */
    abstract summarize(fullTree?: boolean, trackState?: boolean): Promise<ISummaryTreeWithStats>;
    /**
     * {@inheritDoc (ISharedObject:interface).getGCData}
     */
    abstract getGCData(fullGC?: boolean): IGarbageCollectionData;
    /**
     * Called when a handle is decoded by this object. A handle in the object's data represents an outbound reference
     * to another object in the container.
     * @param decodedHandle - The handle of the Fluid object that is decoded.
     */
    protected handleDecoded(decodedHandle: IFluidHandle): void;
    /**
     * Allows the distributed data type to perform custom loading
     * @param services - Storage used by the shared object
     */
    protected abstract loadCore(services: IChannelStorageService): Promise<void>;
    /**
     * Allows the distributed data type to perform custom local loading.
     */
    protected initializeLocalCore(): void;
    /**
     * Allows the distributed data type the ability to perform custom processing once an attach has happened.
     */
    protected abstract registerCore(): any;
    /**
     * Allows the distributive data type the ability to perform custom processing once an attach has happened.
     * Also called after non-local data type get loaded.
     */
    protected didAttach(): void;
    /**
     * Derived classes must override this to do custom processing on a remote message.
     * @param message - The message to process
     * @param local - True if the shared object is local
     * @param localOpMetadata - For local client messages, this is the metadata that was submitted with the message.
     * For messages from a remote client, this will be undefined.
     */
    protected abstract processCore(message: ISequencedDocumentMessage, local: boolean, localOpMetadata: unknown): any;
    /**
     * Called when the object has disconnected from the delta stream.
     */
    protected abstract onDisconnect(): any;
    /**
     * Submits a message by the local client to the runtime.
     * @param content - Content of the message
     * @param localOpMetadata - The local metadata associated with the message. This is kept locally by the runtime
     * and not sent to the server. This will be sent back when this message is received back from the server. This is
     * also sent if we are asked to resubmit the message.
     */
    protected submitLocalMessage(content: any, localOpMetadata?: unknown): void;
    /**
     * Marks this object as dirty so that it is part of the next summary. It is called by a SharedSummaryBlock
     * that want to be part of summary but does not generate ops.
     */
    protected dirty(): void;
    /**
     * Called when the object has fully connected to the delta stream
     * Default implementation for DDS, override if different behavior is required.
     */
    protected onConnect(): void;
    /**
     * Called when a message has to be resubmitted. This typically happens after a reconnection for unacked messages.
     * The default implementation here is to resubmit the same message. The client can override if different behavior
     * is required. It can choose to resubmit the same message, submit different / multiple messages or not submit
     * anything at all.
     * @param content - The content of the original message.
     * @param localOpMetadata - The local metadata associated with the original message.
     */
    protected reSubmitCore(content: any, localOpMetadata: unknown): void;
    /**
     * Promises that are waiting for an ack from the server before resolving should use this instead of new Promise.
     * It ensures that if something changes that will interrupt that ack (e.g. the FluidDataStoreRuntime disposes),
     * the Promise will reject.
     * If runtime is disposed when this call is made, executor is not run and promise is rejected right away.
     */
    protected newAckBasedPromise<T>(executor: (resolve: (value: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void): Promise<T>;
    private attachDeltaHandler;
    /**
     * Set the state of connection to services.
     * @param connected - true if connected, false otherwise.
     */
    private setConnectionState;
    /**
     * Handles a message being received from the remote delta server.
     * @param message - The message to process
     * @param local - Whether the message originated from the local client
     * @param localOpMetadata - For local client messages, this is the metadata that was submitted with the message.
     * For messages from a remote client, this will be undefined.
     */
    private process;
    /**
     * Called when a message has to be resubmitted. This typically happens for unacked messages after a
     * reconnection.
     * @param content - The content of the original message.
     * @param localOpMetadata - The local metadata associated with the original message.
     */
    private reSubmit;
    protected abstract applyStashedOp(content: any): unknown;
}
/**
 * SharedObject with simplified, synchronous summarization and GC.
 * DDS implementations with async and incremental summarization should extend SharedObjectCore directly instead.
 */
export declare abstract class SharedObject<TEvent extends ISharedObjectEvents = ISharedObjectEvents> extends SharedObjectCore<TEvent> {
    /**
     * True while we are garbage collecting this object's data.
     */
    private _isGCing;
    /**
     * The serializer to use to serialize / parse handles, if any.
     */
    private readonly _serializer;
    protected get serializer(): IFluidSerializer;
    /**
     * @param id - The id of the shared object
     * @param runtime - The IFluidDataStoreRuntime which contains the shared object
     * @param attributes - Attributes of the shared object
     */
    constructor(id: string, runtime: IFluidDataStoreRuntime, attributes: IChannelAttributes);
    /**
     * {@inheritDoc (ISharedObject:interface).getAttachSummary}
     */
    getAttachSummary(fullTree?: boolean, trackState?: boolean): ISummaryTreeWithStats;
    /**
     * {@inheritDoc (ISharedObject:interface).summarize}
     */
    summarize(fullTree?: boolean, trackState?: boolean): Promise<ISummaryTreeWithStats>;
    /**
     * {@inheritDoc (ISharedObject:interface).getGCData}
     */
    getGCData(fullGC?: boolean): IGarbageCollectionData;
    /**
     * Calls the serializer over all data in this object that reference other GC nodes.
     * Derived classes must override this to provide custom list of references to other GC nodes.
     */
    protected processGCDataCore(serializer: SummarySerializer): void;
    /**
     * Gets a form of the object that can be serialized.
     * @returns A tree representing the snapshot of the shared object.
     */
    protected abstract summarizeCore(serializer: IFluidSerializer): ISummaryTreeWithStats;
}
//# sourceMappingURL=sharedObject.d.ts.map